#include "SessionManagerWidget.h"
#include "ConfigureManager.h"
#include "RegisterWidget.h"
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/exception.hpp>
#include "CSmtp.h"



using namespace mysqlpp;

bool SessionManagerWidget::m_bPathInitialized = false;
SessionManagerWidget::StringPathMap  SessionManagerWidget::stringPathMap;
SessionManagerWidget::PathStringMap  SessionManagerWidget::pathStringMap;

SessionManagerWidget::StringPathPair SessionManagerWidget::stringPathPair[] =
{
    {"/login"    , LOGIN},
    {"/upload"   , UPLOAD},
    {"/submit"   , SUBMIT},
    {"/register" , REGISTER},
    {"/moniter"  , MONITOR},
    //{"/unknown"  , UNKNOWN},
};

SessionManagerWidget::PathStringPair SessionManagerWidget::pathStringPair[] =
{
    {LOGIN     , "/login"   },
    {UPLOAD    , "/upload"  },
    {SUBMIT    , "/submit"  },
    {REGISTER  ,"/register" },
    {MONITOR   , "/moniter" },
    //{UNKNOWN   , "/unknown" },
};

void SessionManagerWidget::Initialize()
{
    InitializeMainMenu();
    InitializePath();
}

void SessionManagerWidget::InitializeMainMenu()
{
    m_mainMenu = new MainMenuWidget(root());
    m_login = new LoginWidget();
    m_mainMenu->addItem("login", m_login);
    m_register = new RegisterWidget();
    m_mainMenu->addItem("register", m_register);
    m_upload = new UploadFileWidget();
    m_mainMenu->addItem("upload", m_upload);
    m_submit = new SubmitWidget();
    m_mainMenu->addItem("submit", m_submit);
    m_monitor = new TaskViewWidget();
    m_mainMenu->addItem("monitor", m_monitor);
    m_myPath = WApplication::instance()->internalPath();

    m_mainMenu->setHiddenByName("upload", true);
    m_mainMenu->setHiddenByName("submit", true);
    m_mainMenu->setHiddenByName("monitor", true);
    m_login->setupNotifyEventMethod((NotifyMethod)&SessionManagerWidget::EventListener, this);
    m_login->setupValidLoginMethod((LoginWidget::ValidLoginMethod)&SessionManagerWidget::SessionManagerWidget::VerifyUserPassword, this);
    m_register->setupNotifyEventMethod((NotifyMethod)&SessionManagerWidget::EventListener, this);
    m_register->setupValidRegisterMethod((RegisterWidget::ValidRegisterMethod)&SessionManagerWidget::RegisterUserPassword, this);
    m_upload->setupNotifyEventMethod((NotifyMethod)&SessionManagerWidget::EventListener, this);
    m_submit->setupNotifyEventMethod((NotifyMethod)&SessionManagerWidget::EventListener, this);
}

void SessionManagerWidget::InitializePath()
{
    if (! m_bPathInitialized)
    {
        size_t i;
        for (i = 0; i < sizeof(stringPathPair)/sizeof(StringPathPair); i ++)
        {
            stringPathMap.insert(stringPathPair[i]);
        }

        for (i = 0; i < sizeof(PathStringPair)/sizeof(PathStringPair); i ++)
        {
            pathStringMap.insert(pathStringPair[i]);
        }
        m_bPathInitialized = true;
    }
}

int SessionManagerWidget::VerifyUserPassword(const std::string& userName, const std::string& password)
{
    MySqlCon mySqlCon;
    return mySqlCon.VerifyUserPassword(userName, password);
}

bool SessionManagerWidget::RegisterUserPassword(const std::string& userName, const std::string& password, const std::string& email)
{
    MySqlCon mySqlCon;
    return mySqlCon.RegisterUserPassword(userName, password, email);
}


SessionManagerWidget::InternalPathEnum SessionManagerWidget::path2Enum(const std::string& path)
{
    StringPathMap::const_iterator it = stringPathMap.find(path);
    if (it != stringPathMap.end())
    {
        return it->second;
    }
    else
    {
        return UNKNOWN;
    }
}

bool SessionManagerWidget::enum2Path(InternalPathEnum choice, std::string& path)
{
    if (choice != UNKNOWN)
    {
        PathStringMap::const_iterator it = pathStringMap.find(choice);
        if (it != pathStringMap.end())
        {
            path = it->second;
            return true;
        }
    }
    return false;
}

void SessionManagerWidget::UpdateInternalPath()
{
    std::string path = WApplication::instance()->internalPath();
    InternalPathEnum choice;
    if (StateMachine(path, choice))
    {
        UpdateInternalPathByChoice(choice);
    }
}

void SessionManagerWidget::InitializeTask()
{
    MySqlCon mySqlcon;
    mysqlpp::Query query = mySqlcon.con.query();

    m_monitor->clearModel();
    query << "select TaskId from Tasks_View where UserName="<<quote_only<<m_userName;
    FILE_LOG(logDEBUG)<<"InitializeUser:"<<query;
    StoreQueryResult res = query.store();
    for (int i = 0; i < res.size(); i ++)
    {
        int nTaskId = boost::lexical_cast<int>(res[i][0]);
        m_monitor->setTaskId(nTaskId);
    }
}

void SessionManagerWidget::InitializeUser()
{
    m_bLogin = true;
    m_login->getUserName(m_userName);
    m_login->getUserPass(m_userPass);
    m_userId = m_login->getUserId();
    m_userUploadFolder = CloudResourceManager::ConfigureManager::AppSettings("UserUploadFolder") + "/" + m_userName;
    boost::filesystem::path dir(m_userUploadFolder);
    if (!boost::filesystem::exists(dir))
    {
        try
        {
            boost::filesystem::create_directory(dir);
        }
        catch (std::exception e)
        {
            FILE_LOG(logERROR)<<"SessionManagerWidget::InitializeUser:create directory failed:"<<m_userUploadFolder;
            return;
        }
    }
    m_upload->setTargetPath(m_userUploadFolder);
    m_submit->setFolderName(m_userUploadFolder);
    m_submit->setUserId(m_userId);
    InitializeTask();

    m_mainMenu->setHiddenByName("upload", false);
    m_mainMenu->setHiddenByName("submit", false);
    m_mainMenu->setHiddenByName("monitor", false);
}

void SessionManagerWidget::EventListener(NotifyEvent event)
{
    switch (event)
    {
    case LoginSuccessful:
        InitializeUser();
        break;
    case LoginFailed:
        m_bLogin = false;
        m_mainMenu->setHiddenByName("upload", true);
        m_mainMenu->setHiddenByName("submit", true);
        m_mainMenu->setHiddenByName("monitor", true);
        break;
    case UploadSuccessful:
        m_submit->setFolderName(m_userUploadFolder);
        break;
    case SubmitSuccessful:
        InitializeTask();
        break;
    case RegisterSuccessful:
        SendoutActivation();
        break;
    }
}


void SessionManagerWidget::SendoutActivation()
{

    m_register->getUserInfo(m_userName, m_userPass, m_userMail);
    try
    {
        CSmtp mail(true);

        mail.SetSMTPServer("smtp.gmail.com", 465);

        mail.SetSecurityType(CSmtp::USE_SSL);

        mail.SetLogin("nickhuang2009");
        mail.SetPassword("renderingmax");
        mail.SetSenderName("renderingmax");
        mail.SetSenderMail("nickhuang2009@gmail.com");
        mail.SetReplyTo("nickhuang2009@gmail.com");
        mail.SetSubject("re: activate your renderingmax account now!");

        mail.SetXPriority(XPRIORITY_NORMAL);
        mail.SetXMailer("Modified by Nick");
        std::string msg;
        msg += "<!DOCTYPE HTML><html><head>";
        msg += "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">";

        msg += "<title>Activate your renderingmax account today!</title><body>";
        msg += "Dear Mr./Ms. " + m_userName + "(login user " + m_userName + "),<br />";

        msg += "<a href=\"http://192.168.0.7:8080/login?user=" + m_userName + "&pass=" + m_userPass + "\"";
        msg += " style=\"color:#0055aa; text-decoration:underline; padding-right:1px;margin-left:1px;\">";
        msg += "click here to activate your renderingmax account now!</a>";
        msg += "<br />B.Rgds. RenderingMax</body></html>";


        mail.AddRecipient(m_userMail.c_str());
        mail.AddMsgLine(msg.c_str());

        mail.Send();
    }
    catch(ECSmtp e)
    {
        std::cout << "Error: " << e.GetErrorText().c_str() << ".\n";
    }
}

bool SessionManagerWidget::StateMachine(const std::string& path, InternalPathEnum& result)
{
    InternalPathEnum current = path2Enum(m_myPath);
    InternalPathEnum target = path2Enum(path);

    if (m_bLogin)
    {
        if (target == UNKNOWN)
        {
            result = LOGIN;
        }
        else
        {
            result = target;
        }
    }
    else
    {
       result = LOGIN;
    }
    return current != target && target == result;
}

void SessionManagerWidget::UpdateInternalPathByChoice(InternalPathEnum choice)
{
    if (enum2Path(choice, m_myPath))
    {
        WApplication::instance()->setInternalPath(m_myPath, false);

        m_mainMenu->selectByName(m_myPath.substr(1));
    }
}
